@@ -48,6 +48,7 @@
 
 from . import dcc_control
 from . import common
+from . import file_interface
 
 from tkinter import *
 import enum
@@ -109,9 +110,8 @@ def change_button_event (point_id:int):
     return ()
 
 # -------------------------------------------------------------------------
-# Function to flip the state of the Point's Facing Point Lock - called when
-# the FPL button is pressed - Can also be called by the external programme
-# to enable automated route setting functions
+# Public API Function to flip the state of the Point's Facing Point Lock (to
+# enable route setting functions. Also called whenthe FPL button is pressed 
 # -------------------------------------------------------------------------
 
 def toggle_fpl (point_id:int):
@@ -139,11 +139,42 @@ def toggle_fpl (point_id:int):
     return()
 
 # -------------------------------------------------------------------------
-# Function to flip the route setting for the Point - called when the main
-# point button is pressed - Can also be called by the external programme
-# to enable automated route setting functions.
-# Will also recursivelly call itself to change the "also_switch"
-# point to switch if one was specified when the point was created
+# Internal Function to toggle the point blade drawing objects and update
+# the internal state of the point - called by the toggle_point function
+# Can also be called on point creation to set the initial (loaded) state
+# -------------------------------------------------------------------------
+
+def toggle_point_state (point_id:int, switched_by_another_point = False):
+
+    global points
+    global logging
+
+    if not points[str(point_id)]["switched"]:
+        if switched_by_another_point:
+            logging.info ("Point "+str(point_id)+": Changing point to SWITCHED (switched with another point)")
+        else:
+            logging.info ("Point "+str(point_id)+": Changing point to SWITCHED")
+        points[str(point_id)]["changebutton"].config(relief="sunken",bg="white")
+        points[str(point_id)]["switched"] = True
+        points[str(point_id)]["canvas"].itemconfig(points[str(point_id)]["blade2"],state="normal") #switched
+        points[str(point_id)]["canvas"].itemconfig(points[str(point_id)]["blade1"],state="hidden") #normal
+        dcc_control.update_dcc_point(point_id,True)
+    else:
+        if switched_by_another_point:
+            logging.info ("Point "+str(point_id)+": Changing point to NORMAL (switched with another point)")
+        else:
+            logging.info ("Point "+str(point_id)+": Changing point to NORMAL")
+        points[str(point_id)]["changebutton"].config(relief="raised",bg="grey85") 
+        points[str(point_id)]["switched"] = False
+        points[str(point_id)]["canvas"].itemconfig(points[str(point_id)]["blade2"],state="hidden") #switched 
+        points[str(point_id)]["canvas"].itemconfig(points[str(point_id)]["blade1"],state="normal") #normal
+        dcc_control.update_dcc_point(point_id,False)
+    return
+
+# -------------------------------------------------------------------------
+# Public API Function to flip the route setting for the Point (to enable
+# route setting functions. Also called whenthe POINT button is pressed 
+# Will also recursivelly call itself to change any "also_switch" points
 # -------------------------------------------------------------------------
 
 def toggle_point (point_id:int, switched_by_another_point = False):
@@ -160,26 +191,8 @@ def toggle_point (point_id:int, switched_by_another_point = False):
             logging.warning ("Point "+str(point_id)+": Toggle Point - Point is externally locked - Toggling anyway")
         elif points[str(point_id)]["hasfpl"] and points[str(point_id)]["fpllock"]:
             logging.warning ("Point "+str(point_id)+": Toggle Point - Facing Point Lock is active - Toggling anyway")
-        if not points[str(point_id)]["switched"]:
-            if switched_by_another_point:
-                logging.info ("Point "+str(point_id)+": Changing point to SWITCHED (switched with another point)")
-            else:
-                logging.info ("Point "+str(point_id)+": Changing point to SWITCHED")
-            points[str(point_id)]["changebutton"].config(relief="sunken",bg="white")
-            points[str(point_id)]["switched"] = True
-            points[str(point_id)]["canvas"].itemconfig(points[str(point_id)]["blade2"],state="normal") #switched
-            points[str(point_id)]["canvas"].itemconfig(points[str(point_id)]["blade1"],state="hidden") #normal
-            dcc_control.update_dcc_point(point_id,True)
-        else:
-            if switched_by_another_point:
-                logging.info ("Point "+str(point_id)+": Changing point to NORMAL (switched with another point)")
-            else:
-                logging.info ("Point "+str(point_id)+": Changing point to NORMAL")
-            points[str(point_id)]["changebutton"].config(relief="raised",bg="grey85") 
-            points[str(point_id)]["switched"] = False
-            points[str(point_id)]["canvas"].itemconfig(points[str(point_id)]["blade2"],state="hidden") #switched 
-            points[str(point_id)]["canvas"].itemconfig(points[str(point_id)]["blade1"],state="normal") #normal
-            dcc_control.update_dcc_point(point_id,False)
+        # Call the internal function to toggle the point state and update the drawing objects
+        toggle_point_state (point_id,switched_by_another_point)
         # Now change any other points we need (i.e. points switched with this one)
         if points[str(point_id)]["alsoswitch"] != 0:
             if not point_exists(points[str(point_id)]["alsoswitch"]):
@@ -194,11 +207,9 @@ def toggle_point (point_id:int, switched_by_another_point = False):
     return()
 
 # -------------------------------------------------------------------------
-# Externally called function to create a Point (drawing objects + state)
+# Public API function to create a Point (drawing objects + state)
 # By default the point is "NOT SWITCHED" (i.e. showing the default route)
 # If the point has a Facing Point Lock then this is set to locked
-# All attributes (that need to be tracked) are stored as a dictionary
-# This is then added to a dictionary of Points for later reference
 # Function returns a list of the lines that have been drawn (so an
 # external programme can change the colours if required)
 # -------------------------------------------------------------------------
@@ -322,8 +333,21 @@ def create_point (canvas, point_id:int, pointtype:point_type,
 
         # Add the new point to the dictionary of points
         points[str(point_id)] = new_point
-        # Set the initial state of the point
-        dcc_control.update_dcc_point(point_id,False)
+        
+        # Get the initial state for the point (if layout state has been successfully loaded)
+        # if nothing has been loaded then the default state (as created) will be applied
+        loaded_state = file_interface.get_initial_point_state(point_id)
+        # Toggle the FPL if FPL is ACTIVE ("switched" will be 'None' if no data was loaded)
+        # We toggle on False as points with FPLs are created with the FPL active by default
+        if fpl and loaded_state["fpllock"] == False: toggle_fpl(point_id)
+        # Toggle the point state if SWITCHED ("switched" will be 'None' if no data was loaded)
+        # Note that Toggling the point will also send the DCC commands to set the initial state
+        # If we don't toggle the point we need to send out the DCC commands for the default state
+        if loaded_state["switched"]: toggle_point_state(point_id)
+        else: dcc_control.update_dcc_point(point_id,False)
+        # Externally lock the point if required
+        if loaded_state["locked"]: lock_point(point_id)
+
         # We'll also return a list of identifiers for the drawing objects
         # so we can change the colour of them later if required
         # [blade straight, blade switched, route straight, route switched]
@@ -332,9 +356,9 @@ def create_point (canvas, point_id:int, pointtype:point_type,
     return(point_objects)
 
 # -------------------------------------------------------------------------
-# Externally called function to Lock one or more points. If the external
-# signal/point locking code has been correctly implemented it should only
-# be possible to lock a point that has the Facing point Lock activated
+# Public API function to Lock one or more points. The external signal/point
+# interlocking should be written to ensure it is only possible to lock a
+# point when the Facing point Lock is activated
 # -------------------------------------------------------------------------
 
 def lock_point (*point_ids:int):
@@ -359,7 +383,7 @@ def lock_point (*point_ids:int):
     return()
 
 # -------------------------------------------------------------------------
-# Externally called function to Unlock one or more points
+# Public API function to Unlock one or more points
 # -------------------------------------------------------------------------
 
 def unlock_point (*point_ids:int):
@@ -381,7 +405,7 @@ def unlock_point (*point_ids:int):
     return ()
 
 # -------------------------------------------------------------------------
-# Externally called function to Return the current state of the point
+# Public API function to Return the current state of the point
 # -------------------------------------------------------------------------
 
 def point_switched (point_id:int):
@@ -395,7 +419,7 @@ def point_switched (point_id:int):
     return(switched)
 
 # -------------------------------------------------------------------------
-# Externally called function to query the current state of the FPL
+# Public API function to query the current state of the FPL
 # if the point does not have a FPL the return will always be TRUE
 # -------------------------------------------------------------------------
 
